==> virtualfs-1.32.src.tar.gz: virtualfs-1.32/litefs/bigcp.cc <==
// This is a small cp utility that copies using large chunk (30000).
// This speeds up litefs a lot over slow link. At some point, litefs-proxy
// will learn how to do write-behind and read-ahead, but in the mean
// time, this should help the speed quite a lot
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utime.h>

static void usage()
{
	fprintf (stderr
			,"bigcp [options] source-file dest-file\n"
			"bigcp [options] source-files .... dest-dir\n"
			"\n"
			"-v: verbose mode"
			"\n");
}

static int copy_file (const char *src, const char *dst, int verbose)
{
	int ret = -1;
	FILE *fin = fopen (src,"r");
	if (fin == NULL){
		fprintf (stderr,"Can't open file %s (%s)\n",src,strerror(errno));
	}else{
		struct stat st;
		char dsttmp[PATH_MAX];
		if (stat(dst,&st)!=-1 && S_ISDIR(st.st_mode)){			 
			const char *endfile = strrchr(src,'/');
			if (endfile == NULL) endfile = src;
			snprintf (dsttmp,sizeof(dsttmp)-1,"%s/%s",dst,endfile);
			dst = dsttmp;
		}
		FILE *fout = fopen (dst,"w");
		if (fout == NULL){
			fprintf (stderr,"Can't open file %s (%s)\n",dst,strerror(errno));
		}else{
			char buf[32000];
			int n;
			if (verbose) printf ("%s:",src);
			while ((n=fread(buf,1,32000,fin))> 0){
				if (verbose) printf (".");
				fwrite (buf,1,n,fout);
			}
			ret = fclose (fout);
			if (ret == 0){
				if (verbose) printf (" ok");
				struct stat st;
				if (stat(src,&st)!=-1){
					struct utimbuf tim;
					tim.actime = st.st_atime;
					tim.modtime = st.st_mtime;
					ret = utime (dst,&tim);
				}
			}
			if (verbose) printf ("\n");
		}
		fclose (fin);
	}
	return ret;
}

int main (int argc, char *argv[])
{
	int ret = -1;
	int verbose = 0;
	int firstarg;
	for (firstarg=1; firstarg<argc; firstarg++){
		char *arg = argv[firstarg];
		if (arg[0] == '-'){
			if (strcmp(arg,"-v")==0){
				setbuf (stdout,NULL);
				verbose++;
			}else{
				usage();
				exit (-1);
			}
		}else{
			break;
		}
	}
	if (argc - firstarg < 2){
		usage();
	}else if (argc - firstarg == 2){
		ret = copy_file (argv[firstarg],argv[firstarg+1],verbose);
	}else{
		// More than one file, the target has to be a directory
		const char *dir = argv[argc-1];
		for (int i=firstarg; i<argc-1; i++){
			char path[PATH_MAX];
			const char *file = argv[i];
			const char *endfile = strrchr(file,'/');
			if (endfile == NULL) endfile = file;
			snprintf (path,sizeof(path)-1,"%s/%s",dir,endfile);
			ret = copy_file (file,path,verbose);
			if (ret == -1) break;
		}
	}
	return ret;
}

==> virtualfs-1.38.src.tar.gz: virtualfs-1.38/litefs/bigcp.cc <==
// This is a small cp utility that copies using large chunk (30000).
// This speeds up litefs a lot over slow link. At some point, litefs-proxy
// will learn how to do write-behind and read-ahead, but in the mean
// time, this should help the speed quite a lot
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utime.h>
#include <ctype.h>

static void usage()
{
	fprintf (stderr
			,"bigcp version %s\n"
			"bigcp [options] source-file dest-file\n"
			"bigcp [options] source-files .... dest-dir\n"
			"\n"
			"-v: verbose mode"
			"\n"
		,PACKAGE_REV);
}

static int copy_file (
	const char *src,
	const char *dst,
	int verbose,
	int blksize)
{
	int ret = -1;
	FILE *fin = fopen (src,"r");
	if (fin == NULL){
		fprintf (stderr,"Can't open file %s (%s)\n",src,strerror(errno));
	}else{
		struct stat st;
		char dsttmp[PATH_MAX];
		if (stat(dst,&st)!=-1 && S_ISDIR(st.st_mode)){			 
			const char *endfile = strrchr(src,'/');
			if (endfile == NULL) endfile = src;
			snprintf (dsttmp,sizeof(dsttmp)-1,"%s/%s",dst,endfile);
			dst = dsttmp;
		}
		FILE *fout = fopen (dst,"w");
		if (fout == NULL){
			fprintf (stderr,"Can't open file %s (%s)\n",dst,strerror(errno));
		}else{
			setbuf (fout,NULL);
			char buf[blksize];
			int n;
			if (verbose) printf ("%s:",src);
			while ((n=fread(buf,1,blksize,fin))> 0){
				if (verbose) printf (".");
				#if 0
					write (fileno(fout),buf,n);
				#else
					fwrite (buf,1,n,fout);
					// fflush (fout);
				#endif
			}
			ret = fclose (fout);
			if (ret == 0){
				if (verbose) printf (" ok");
				struct stat st;
				if (stat(src,&st)!=-1){
					struct utimbuf tim;
					tim.actime = st.st_atime;
					tim.modtime = st.st_mtime;
					ret = utime (dst,&tim);
				}
			}
			if (verbose) printf ("\n");
		}
		fclose (fin);
	}
	return ret;
}

int main (int argc, char *argv[])
{
	int ret = -1;
	int verbose = 0;
	int blksize = 32000;
	int firstarg;
	for (firstarg=1; firstarg<argc; firstarg++){
		char *arg = argv[firstarg];
		if (arg[0] == '-'){
			if (strcmp(arg,"-v")==0){
				setbuf (stdout,NULL);
				verbose++;
			}else if (strncmp(arg,"-b",2)==0 && isdigit(arg[2])){
				blksize = atoi(arg+2);
			}else if (strcmp(arg,"-b")==0 && argc > firstarg+1){
				firstarg++;
				blksize = atoi(argv[firstarg]);
			}else{
				usage();
				exit (-1);
			}
		}else{
			break;
		}
	}
	if (argc - firstarg < 2){
		usage();
	}else if (argc - firstarg == 2){
		ret = copy_file (argv[firstarg],argv[firstarg+1],verbose,blksize);
	}else{
		// More than one file, the target has to be a directory
		const char *dir = argv[argc-1];
		for (int i=firstarg; i<argc-1; i++){
			char path[PATH_MAX];
			const char *file = argv[i];
			const char *endfile = strrchr(file,'/');
			if (endfile == NULL) endfile = file;
			snprintf (path,sizeof(path)-1,"%s/%s",dir,endfile);
			ret = copy_file (file,path,verbose,blksize);
			if (ret == -1) break;
		}
	}
	return ret;
}
